@@ -5,7 +5,7 @@ set -o errexit
 test_import () {
     echo "Create environment: python=$PYTHON_VERSION $1"
     # Create an empty environment
-    mamba create -q -y -n test-imports -c conda-forge python=$PYTHON_VERSION packaging pyyaml fsspec toolz partd cloudpickle $1
+    mamba create -q -y -n test-imports -c conda-forge python=$PYTHON_VERSION packaging pyyaml fsspec toolz partd click cloudpickle $1
     conda activate test-imports
     if [[ $1 =~ "distributed" ]]; then
         # dask[distributed] depends on the latest version of distributed
@@ -15,6 +15,10 @@ test_import () {
     mamba list
     echo "python -c '$2'"
     python -c "$2"
+    # Ensure that no non-deterministic objects are tokenized at init time,
+    # which can prevent the library from being imported at all.
+    echo "python -c '$2' (ensure deterministic)"
+    DASK_TOKENIZE__ENSURE_DETERMINISTIC=True python -c "$2"
     conda deactivate
     mamba env remove -n test-imports
 }
