@@ -3,10 +3,10 @@ import warnings
 
 import tlz as toolz
 
-from .. import base, utils
-from ..blockwise import blockwise as core_blockwise
-from ..delayed import unpack_collections
-from ..highlevelgraph import HighLevelGraph
+from dask import base, utils
+from dask.blockwise import blockwise as core_blockwise
+from dask.delayed import unpack_collections
+from dask.highlevelgraph import HighLevelGraph
 
 
 def blockwise(
@@ -49,6 +49,13 @@ def blockwise(
         Dictionary mapping index to function to be applied to chunk sizes
     new_axes : dict, keyword only
         New indexes and their dimension lengths
+    align_arrays: bool
+        Whether or not to align chunks along equally sized dimensions when
+        multiple arrays are provided.  This allows for larger chunks in some
+        arrays to be broken into smaller ones that match chunk sizes in other
+        arrays such that they are compatible for block function mapping. If
+        this is false, then an error will be thrown if arrays do not already
+        have the same number of blocks in each dimension.
 
     Examples
     --------
@@ -168,7 +175,7 @@ def blockwise(
     if new:
         raise ValueError("Unknown dimension", new)
 
-    from .core import normalize_arg, unify_chunks
+    from dask.array.core import normalize_arg, unify_chunks
 
     if align_arrays:
         chunkss, arrays = unify_chunks(*args)
@@ -271,7 +278,7 @@ def blockwise(
     chunks = tuple(chunks)
 
     if meta is None:
-        from .utils import compute_meta
+        from dask.array.utils import compute_meta
 
         meta = compute_meta(func, dtype, *args[::2], **kwargs)
     return new_da_object(graph, out, chunks, meta=meta, dtype=dtype)
@@ -282,4 +289,4 @@ def atop(*args, **kwargs):
     return blockwise(*args, **kwargs)
 
 
-from .core import new_da_object
+from dask.array.core import new_da_object
