@@ -1,17 +1,26 @@
-from itertools import product
 import warnings
+from itertools import product
 
 import pytest
 
 np = pytest.importorskip("numpy")
 
 import dask
-from dask.utils import funcname
-from dask.array.utils import assert_eq
-from dask.array.rechunk import intersect_chunks, rechunk, normalize_chunks
-from dask.array.rechunk import cumdims_label, _breakpoints, _intersect_1d, _old_to_new
-from dask.array.rechunk import plan_rechunk, divide_to_width, merge_to_number
 import dask.array as da
+from dask.array.rechunk import (
+    _breakpoints,
+    _intersect_1d,
+    _old_to_new,
+    cumdims_label,
+    divide_to_width,
+    intersect_chunks,
+    merge_to_number,
+    normalize_chunks,
+    plan_rechunk,
+    rechunk,
+)
+from dask.array.utils import assert_eq
+from dask.utils import funcname
 
 
 def test_rechunk_internals_1():
@@ -51,9 +60,40 @@ def test_rechunk_internals_1():
     ]
     assert i1d[1] == answer4
 
+    new = cumdims_label(((1, 1, 2), (1, 5, 1, 0)), "n")
+    breaks = tuple(_breakpoints(o, n) for o, n in zip(old, new))
+    answer5 = (
+        ("o", 0),
+        ("n", 0),
+        ("o", 1),
+        ("n", 1),
+        ("o", 2),
+        ("o", 3),
+        ("o", 4),
+        ("o", 5),
+        ("n", 6),
+        ("n", 7),
+        ("n", 7),
+    )
+    assert breaks[1] == answer5
+    i1d = [_intersect_1d(b) for b in breaks]
+    answer6 = [
+        [(0, slice(0, 1))],
+        [
+            (1, slice(0, 1)),
+            (2, slice(0, 1)),
+            (3, slice(0, 1)),
+            (4, slice(0, 1)),
+            (5, slice(0, 1)),
+        ],
+        [(5, slice(1, 2))],
+        [(5, slice(2, 2))],
+    ]
+    assert i1d[1] == answer6
+
 
 def test_intersect_1():
-    """ Convert 1 D chunks"""
+    """Convert 1 D chunks"""
     old = ((10, 10, 10, 10, 10),)
     new = ((25, 5, 20),)
     answer = [
@@ -66,7 +106,7 @@ def test_intersect_1():
 
 
 def test_intersect_2():
-    """ Convert 1 D chunks"""
+    """Convert 1 D chunks"""
     old = ((20, 20, 20, 20, 20),)
     new = ((58, 4, 20, 18),)
     answer = [
@@ -119,7 +159,7 @@ def test_rechunk_expand():
 
 def test_rechunk_expand2():
     (a, b) = (3, 2)
-    orig = np.random.uniform(0, 1, a ** b).reshape((a,) * b)
+    orig = np.random.uniform(0, 1, a**b).reshape((a,) * b)
     for off, off2 in product(range(1, a - 1), range(1, a - 1)):
         old = ((a - off, off),) * b
         x = da.from_array(orig, chunks=old)
@@ -132,7 +172,7 @@ def test_rechunk_expand2():
 
 
 def test_rechunk_method():
-    """ Test rechunking can be done as a method of dask array."""
+    """Test rechunking can be done as a method of dask array."""
     old = ((5, 2, 3),) * 4
     new = ((3, 3, 3, 1),) * 4
     a = np.random.uniform(0, 1, 10000).reshape((10,) * 4)
@@ -143,7 +183,7 @@ def test_rechunk_method():
 
 
 def test_rechunk_blockshape():
-    """ Test that blockshape can be used."""
+    """Test that blockshape can be used."""
     new_shape, new_chunks = (10, 10), (4, 3)
     new_blockdims = normalize_chunks(new_chunks, new_shape)
     old_chunks = ((4, 4, 2), (3, 3, 3, 1))
@@ -172,6 +212,10 @@ def test_rechunk_with_dict():
     y = x.rechunk(chunks={0: -1})
     assert y.chunks == ((24,), (8, 8, 8))
 
+    x = da.ones((24, 24), chunks=(4, 8))
+    y = x.rechunk(chunks={0: None, 1: "auto"})
+    assert y.chunks == ((4, 4, 4, 4, 4, 4), (24,))
+
 
 def test_rechunk_with_empty_input():
     x = da.ones((24, 24), chunks=(4, 8))
@@ -182,6 +226,10 @@ def test_rechunk_with_empty_input():
 def test_rechunk_with_null_dimensions():
     x = da.from_array(np.ones((24, 24)), chunks=(4, 8))
     assert x.rechunk(chunks=(None, 4)).chunks == da.ones((24, 24), chunks=(4, 4)).chunks
+    assert (
+        x.rechunk(chunks={0: None, 1: 4}).chunks
+        == da.ones((24, 24), chunks=(4, 4)).chunks
+    )
 
 
 def test_rechunk_with_integer():
@@ -725,12 +773,15 @@ def test_rechunk_auto_image_stack(n):
 
     with dask.config.set({"array.chunk-size": "7MiB"}):
         z = x.rechunk("auto")
-        assert z.chunks == ((5,) * (n // 5), (1000,), (1000,))
+        if n == 100:
+            assert z.chunks == ((7,) * 14 + (2,), (1000,), (1000,))
+        else:
+            assert z.chunks == ((7,) * 142 + (6,), (1000,), (1000,))
 
     with dask.config.set({"array.chunk-size": "1MiB"}):
         x = da.ones((n, 1000, 1000), chunks=(1, 1000, 1000), dtype="float64")
         z = x.rechunk("auto")
-        assert z.chunks == ((1,) * n, (250,) * 4, (250,) * 4)
+        assert z.chunks == ((1,) * n, (362, 362, 276), (362, 362, 276))
 
 
 def test_rechunk_down():
@@ -741,14 +792,14 @@ def test_rechunk_down():
 
     with dask.config.set({"array.chunk-size": "1MiB"}):
         z = y.rechunk("auto")
-        assert z.chunks == ((5,) * 20, (250,) * 4, (250,) * 4)
+        assert z.chunks == ((4,) * 25, (511, 489), (511, 489))
 
     with dask.config.set({"array.chunk-size": "1MiB"}):
         z = y.rechunk({0: "auto"})
         assert z.chunks == ((1,) * 100, (1000,), (1000,))
 
         z = y.rechunk({1: "auto"})
-        assert z.chunks == ((10,) * 10, (100,) * 10, (1000,))
+        assert z.chunks == ((10,) * 10, (104,) * 9 + (64,), (1000,))
 
 
 def test_rechunk_zero():
@@ -896,3 +947,129 @@ def test_balance_split_into_n_chunks():
             x = da.from_array(np.random.uniform(size=N))
             y = x.rechunk(chunks=len(x) // nchunks, balance=True)
             assert len(y.chunks[0]) == nchunks
+
+
+def test_rechunk_with_zero():
+    a = da.ones((8, 8), chunks=(4, 4))
+    result = a.rechunk(((4, 4), (4, 0, 0, 4)))
+    expected = da.ones((8, 8), chunks=((4, 4), (4, 0, 0, 4)))
+
+    # reverse:
+    a, expected = expected, a
+    result = a.rechunk((4, 4))
+    assert_eq(result, expected)
+
+
+def test_intersect_chunks_with_nonzero():
+    from dask.array.rechunk import intersect_chunks
+
+    old = ((4, 4), (2,))
+    new = ((8,), (1, 1))
+    result = list(intersect_chunks(old, new))
+    expected = [
+        (
+            ((0, slice(0, 4, None)), (0, slice(0, 1, None))),
+            ((1, slice(0, 4, None)), (0, slice(0, 1, None))),
+        ),
+        (
+            ((0, slice(0, 4, None)), (0, slice(1, 2, None))),
+            ((1, slice(0, 4, None)), (0, slice(1, 2, None))),
+        ),
+    ]
+    assert result == expected
+
+
+def test_intersect_chunks_with_zero():
+    from dask.array.rechunk import intersect_chunks
+
+    old = ((4, 4), (2,))
+    new = ((4, 0, 0, 4), (1, 1))
+    result = list(intersect_chunks(old, new))
+
+    expected = [
+        (((0, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 4, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 4, None)), (0, slice(1, 2, None))),),
+    ]
+
+    assert result == expected
+
+    old = ((4, 0, 0, 4), (1, 1))
+    new = ((4, 4), (2,))
+    result = list(intersect_chunks(old, new))
+
+    expected = [
+        (
+            ((0, slice(0, 4, None)), (0, slice(0, 1, None))),
+            ((0, slice(0, 4, None)), (1, slice(0, 1, None))),
+        ),
+        (
+            ((3, slice(0, 4, None)), (0, slice(0, 1, None))),
+            ((3, slice(0, 4, None)), (1, slice(0, 1, None))),
+        ),
+    ]
+
+    assert result == expected
+
+    old = ((4, 4), (2,))
+    new = ((2, 0, 0, 2, 4), (1, 1))
+    result = list(intersect_chunks(old, new))
+    expected = [
+        (((0, slice(0, 2, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 2, None)), (0, slice(1, 2, None))),),
+        (((0, slice(2, 2, None)), (0, slice(0, 1, None))),),
+        (((0, slice(2, 2, None)), (0, slice(1, 2, None))),),
+        (((0, slice(2, 2, None)), (0, slice(0, 1, None))),),
+        (((0, slice(2, 2, None)), (0, slice(1, 2, None))),),
+        (((0, slice(2, 4, None)), (0, slice(0, 1, None))),),
+        (((0, slice(2, 4, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 4, None)), (0, slice(1, 2, None))),),
+    ]
+
+    assert result == expected
+
+    old = ((4, 4), (2,))
+    new = ((0, 0, 4, 4), (1, 1))
+    result = list(intersect_chunks(old, new))
+    expected = [
+        (((0, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((0, slice(0, 0, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 0, None)), (0, slice(1, 2, None))),),
+        (((0, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((0, slice(0, 4, None)), (0, slice(1, 2, None))),),
+        (((1, slice(0, 4, None)), (0, slice(0, 1, None))),),
+        (((1, slice(0, 4, None)), (0, slice(1, 2, None))),),
+    ]
+
+    assert result == expected
+
+
+def test_old_to_new_with_zero():
+    from dask.array.rechunk import _old_to_new
+
+    old = ((4, 4),)
+    new = ((4, 0, 4),)
+    result = _old_to_new(old, new)
+    expected = [[[(0, slice(0, 4))], [(1, slice(0, 0))], [(1, slice(0, 4))]]]
+    assert result == expected
+
+    old = ((4,),)
+    new = ((4, 0),)
+    result = _old_to_new(old, new)
+    expected = [[[(0, slice(0, 4))], [(0, slice(4, 4))]]]
+    assert result == expected
+
+    old = ((4, 0, 4),)
+    new = ((4, 0, 2, 2),)
+    result = _old_to_new(old, new)
+    expected = [
+        [[(0, slice(0, 4))], [(2, slice(0, 0))], [(2, slice(0, 2))], [(2, slice(2, 4))]]
+    ]
+    assert result == expected
