@@ -1,16 +1,19 @@
 import contextlib
 import itertools
 import sys
+import warnings
 from numbers import Number
 
 import pytest
 from numpy import AxisError
 
+import dask
 from dask.delayed import delayed
 
 np = pytest.importorskip("numpy")
 
 import dask.array as da
+from dask.array.numpy_compat import _numpy_123
 from dask.array.utils import assert_eq, same_keys
 
 
@@ -242,8 +245,8 @@ def test_rot90(kwargs, shape):
     np_a = np.random.random(shape)
     da_a = da.from_array(np_a, chunks=1)
 
-    np_func = getattr(np, "rot90")
-    da_func = getattr(da, "rot90")
+    np_func = np.rot90
+    da_func = da.rot90
 
     try:
         for axis in axes[:2]:
@@ -392,6 +395,19 @@ def test_dot_method():
     assert_eq(a.dot(b), x.dot(y))
 
 
+def test_dot_persist_equivalence():
+    # Regression test for https://github.com/dask/dask/issues/6907
+    x = da.random.random((4, 4), chunks=(2, 2))
+    x[x < 0.65] = 0
+    y = x.persist()
+    z = x.compute()
+    r1 = da.dot(x, x).compute()
+    r2 = da.dot(y, y).compute()
+    rr = np.dot(z, z)
+    assert np.allclose(rr, r1)
+    assert np.allclose(rr, r2)
+
+
 @pytest.mark.parametrize("shape, chunks", [((20,), (6,)), ((4, 5), (2, 3))])
 def test_vdot(shape, chunks):
     np.random.seed(1337)
@@ -1171,7 +1187,8 @@ def test_cov():
 
     assert_eq(da.cov(d), np.cov(x))
     assert_eq(da.cov(d, rowvar=0), np.cov(x, rowvar=0))
-    with pytest.warns(None):  # warning dof <= 0 for slice
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=RuntimeWarning)  # dof <= 0 for slice
         assert_eq(da.cov(d, ddof=10), np.cov(x, ddof=10))
     assert_eq(da.cov(d, bias=1), np.cov(x, bias=1))
     assert_eq(da.cov(d, d), np.cov(x, x))
@@ -1291,7 +1308,8 @@ def test_isin_rand(
     a2 = rng.randint(low, high, size=test_shape) - 5
     d2 = da.from_array(a2, chunks=test_chunks)
 
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=da.PerformanceWarning)
         r_a = np.isin(a1, a2, invert=invert)
         r_d = da.isin(d1, d2, invert=invert)
     assert_eq(r_a, r_d)
@@ -1329,6 +1347,18 @@ def test_roll(chunks, shift, axis):
         assert_eq(np.roll(x, shift, axis), da.roll(a, shift, axis))
 
 
+def test_roll_always_results_in_a_new_array():
+    x = da.arange(2, 3)
+    y = da.roll(x, 1)
+    assert y is not x
+
+
+def test_roll_works_even_if_shape_is_0():
+    expected = np.roll(np.zeros(0), 0)
+    actual = da.roll(da.zeros(0), 0)
+    assert_eq(expected, actual)
+
+
 @pytest.mark.parametrize("shape", [(10,), (5, 10), (5, 10, 10)])
 def test_shape_and_ndim(shape):
     x = da.random.random(shape)
@@ -1460,6 +1490,17 @@ def test_squeeze(is_func, axis):
     assert d_s.chunks == exp_d_s_chunks
 
 
+@pytest.mark.parametrize("shape", [(1,), (1, 1)])
+def test_squeeze_1d_array(shape):
+    a = np.full(shape=shape, fill_value=2)
+    a_s = np.squeeze(a)
+    d = da.from_array(a, chunks=(1))
+    d_s = da.squeeze(d)
+    assert isinstance(d_s, da.Array)
+    assert isinstance(d_s.compute(), np.ndarray)
+    assert_eq(d_s, a_s)
+
+
 def test_vstack():
     x = np.arange(5)
     y = np.ones(5)
@@ -1990,7 +2031,7 @@ def test_unravel_index():
         for i in range(len(indices)):
             assert_eq(d_indices[i], indices[i])
 
-        assert_eq(darr.vindex[d_indices], arr[indices])
+        assert_eq(darr.vindex[dask.compute(*d_indices)], arr[indices])
 
 
 @pytest.mark.parametrize(
@@ -2138,7 +2179,7 @@ def test_coarsen_bad_chunks(chunks):
 )
 def test_aligned_coarsen_chunks(chunks, divisor):
 
-    from ..routines import aligned_coarsen_chunks as acc
+    from dask.array.routines import aligned_coarsen_chunks as acc
 
     aligned_chunks = acc(chunks, divisor)
     any_remainders = (np.array(aligned_chunks) % divisor) != 0
@@ -2386,7 +2427,8 @@ def test_einsum(einsum_signature):
 
     np_inputs, da_inputs = _numpy_and_dask_inputs(input_sigs)
 
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=da.PerformanceWarning)
         assert_eq(
             np.einsum(einsum_signature, *np_inputs),
             da.einsum(einsum_signature, *da_inputs),
@@ -2524,17 +2566,31 @@ def test_average(a, returned):
     assert_eq(np_avg, da_avg)
 
 
-def test_average_weights():
+@pytest.mark.parametrize("a", [np.arange(11), np.arange(6).reshape((3, 2))])
+def test_average_keepdims(a):
+    d_a = da.from_array(a, chunks=2)
+
+    da_avg = da.average(d_a, keepdims=True)
+
+    if _numpy_123:
+        np_avg = np.average(a, keepdims=True)
+        assert_eq(np_avg, da_avg)
+
+
+@pytest.mark.parametrize("keepdims", [False, True])
+def test_average_weights(keepdims):
     a = np.arange(6).reshape((3, 2))
     d_a = da.from_array(a, chunks=2)
 
     weights = np.array([0.25, 0.75])
     d_weights = da.from_array(weights, chunks=2)
 
-    np_avg = np.average(a, weights=weights, axis=1)
-    da_avg = da.average(d_a, weights=d_weights, axis=1)
+    da_avg = da.average(d_a, weights=d_weights, axis=1, keepdims=keepdims)
 
-    assert_eq(np_avg, da_avg)
+    if _numpy_123:
+        assert_eq(da_avg, np.average(a, weights=weights, axis=1, keepdims=keepdims))
+    elif not keepdims:
+        assert_eq(da_avg, np.average(a, weights=weights, axis=1))
 
 
 def test_average_raises():
