@@ -1,11 +1,10 @@
 try:
-    from ..base import compute
-    from .avro import read_avro
-    from .core import Bag, Item
-    from .core import bag_map as map
-    from .core import bag_range as range
-    from .core import bag_zip as zip
-    from .core import (
+    from dask.bag.avro import read_avro
+    from dask.bag.core import Bag, Item
+    from dask.bag.core import bag_map as map
+    from dask.bag.core import bag_range as range
+    from dask.bag.core import bag_zip as zip
+    from dask.bag.core import (
         concat,
         from_delayed,
         from_sequence,
@@ -13,8 +12,9 @@ try:
         map_partitions,
         to_textfiles,
     )
-    from .text import read_text
-    from .utils import assert_eq
+    from dask.bag.text import read_text
+    from dask.bag.utils import assert_eq
+    from dask.base import compute
 except ImportError as e:
     msg = (
         "Dask bag requirements are not installed.\n\n"
