@@ -1,34 +1,41 @@
+from __future__ import annotations
+
 import math
-import numbers
 import re
 import sys
 import textwrap
 import traceback
 from collections.abc import Iterator, Mapping
 from contextlib import contextmanager
+from numbers import Number
+from typing import Callable, TypeVar, overload
 
 import numpy as np
 import pandas as pd
-from pandas.api.types import is_scalar  # noqa: F401
 from pandas.api.types import is_categorical_dtype, is_dtype_equal
 
-from ..base import is_dask_collection
-from ..core import get_deps
-from ..local import get_sync
-from ..utils import is_arraylike  # noqa: F401
-from ..utils import asciitable
-from ..utils import is_dataframe_like as dask_is_dataframe_like
-from ..utils import is_index_like as dask_is_index_like
-from ..utils import is_series_like as dask_is_series_like
-from ..utils import typename
-from . import _dtypes  # noqa: F401 register pandas extension types
-from . import methods
-from ._compat import PANDAS_GT_110, PANDAS_GT_120, tm  # noqa: F401
-from .dispatch import make_meta  # noqa : F401
-from .dispatch import make_meta_obj, meta_nonempty  # noqa : F401
-from .extensions import make_scalar
+from dask.base import get_scheduler, is_dask_collection
+from dask.core import get_deps
+from dask.dataframe import (  # noqa: F401 register pandas extension types
+    _dtypes,
+    methods,
+)
+from dask.dataframe._compat import PANDAS_GT_110, PANDAS_GT_120, tm  # noqa: F401
+from dask.dataframe.dispatch import (  # noqa : F401
+    make_meta,
+    make_meta_obj,
+    meta_nonempty,
+)
+from dask.dataframe.extensions import make_scalar
+from dask.utils import (
+    asciitable,
+    is_dataframe_like,
+    is_index_like,
+    is_series_like,
+    typename,
+)
 
-meta_object_types = (pd.Series, pd.DataFrame, pd.Index, pd.MultiIndex)
+meta_object_types: tuple[type, ...] = (pd.Series, pd.DataFrame, pd.Index, pd.MultiIndex)
 try:
     import scipy.sparse as sp
 
@@ -134,6 +141,18 @@ infer the metadata. This may lead to unexpected results, so providing
 ``dask.dataframe.utils.make_meta``.
 """
 
+T = TypeVar("T", bound=Callable)
+
+
+@overload
+def insert_meta_param_description(func: T) -> T:
+    ...
+
+
+@overload
+def insert_meta_param_description(pad: int) -> Callable[[T], T]:
+    ...
+
 
 def insert_meta_param_description(*args, **kwargs):
     """Replace `$META` in docstring with param description.
@@ -279,9 +298,10 @@ def clear_known_categories(x, cols=None, index=True):
 
 def _empty_series(name, dtype, index=None):
     if isinstance(dtype, str) and dtype == "category":
-        return pd.Series(
-            pd.Categorical([UNKNOWN_CATEGORIES]), name=name, index=index
-        ).iloc[:0]
+        s = pd.Series(pd.Categorical([UNKNOWN_CATEGORIES]), name=name).iloc[:0]
+        if index is not None:
+            s.index = make_meta(index)
+        return s
     return pd.Series([], dtype=dtype, name=name, index=index)
 
 
@@ -320,18 +340,6 @@ def _nonempty_scalar(x):
     raise TypeError(f"Can't handle meta of type '{typename(type(x))}'")
 
 
-def is_dataframe_like(df):
-    return dask_is_dataframe_like(df)
-
-
-def is_series_like(s):
-    return dask_is_series_like(s)
-
-
-def is_index_like(s):
-    return dask_is_index_like(s)
-
-
 def check_meta(x, meta, funcname=None, numeric_equal=True):
     """Check that the dask metadata matches the result.
 
@@ -445,7 +453,7 @@ def index_summary(idx, name=None):
 ###############################################################
 
 
-def _check_dask(dsk, check_names=True, check_dtypes=True, result=None):
+def _check_dask(dsk, check_names=True, check_dtypes=True, result=None, scheduler=None):
     import dask.dataframe as dd
 
     if hasattr(dsk, "__dask_graph__"):
@@ -453,7 +461,7 @@ def _check_dask(dsk, check_names=True, check_dtypes=True, result=None):
         if hasattr(graph, "validate"):
             graph.validate()
         if result is None:
-            result = dsk.compute(scheduler="sync")
+            result = dsk.compute(scheduler=scheduler)
         if isinstance(dsk, dd.Index):
             assert "Index" in type(result).__name__, type(result)
             # assert type(dsk._meta) == type(result), type(dsk._meta)
@@ -529,24 +537,30 @@ def assert_eq(
     check_dtype=True,
     check_divisions=True,
     check_index=True,
+    sort_results=True,
+    scheduler="sync",
     **kwargs,
 ):
     if check_divisions:
-        assert_divisions(a)
-        assert_divisions(b)
+        assert_divisions(a, scheduler=scheduler)
+        assert_divisions(b, scheduler=scheduler)
         if hasattr(a, "divisions") and hasattr(b, "divisions"):
             at = type(np.asarray(a.divisions).tolist()[0])  # numpy to python
             bt = type(np.asarray(b.divisions).tolist()[0])  # scalar conversion
             assert at == bt, (at, bt)
     assert_sane_keynames(a)
     assert_sane_keynames(b)
-    a = _check_dask(a, check_names=check_names, check_dtypes=check_dtype)
-    b = _check_dask(b, check_names=check_names, check_dtypes=check_dtype)
+    a = _check_dask(
+        a, check_names=check_names, check_dtypes=check_dtype, scheduler=scheduler
+    )
+    b = _check_dask(
+        b, check_names=check_names, check_dtypes=check_dtype, scheduler=scheduler
+    )
     if hasattr(a, "to_pandas"):
         a = a.to_pandas()
     if hasattr(b, "to_pandas"):
         b = b.to_pandas()
-    if isinstance(a, (pd.DataFrame, pd.Series)):
+    if isinstance(a, (pd.DataFrame, pd.Series)) and sort_results:
         a = _maybe_sort(a, check_index)
         b = _maybe_sort(b, check_index)
     if not check_index:
@@ -585,7 +599,7 @@ def assert_dask_graph(dask, label):
     raise AssertionError(f"given dask graph doesn't contain label: {label}")
 
 
-def assert_divisions(ddf):
+def assert_divisions(ddf, scheduler=None):
     if not hasattr(ddf, "divisions"):
         return
 
@@ -602,7 +616,8 @@ def assert_divisions(ddf):
         except AttributeError:
             return x.index
 
-    results = get_sync(ddf.dask, ddf.__dask_keys__())
+    get = get_scheduler(scheduler=scheduler, collections=[type(ddf)])
+    results = get(ddf.dask, ddf.__dask_keys__())
     for i, df in enumerate(results[:-1]):
         if len(df):
             assert index(df).min() >= ddf.divisions[i]
@@ -642,7 +657,9 @@ def assert_dask_dtypes(ddf, res, numeric_equal=True):
         ) or (a == b)
 
     if not is_dask_collection(res) and is_dataframe_like(res):
-        for col, a, b in pd.concat([ddf._meta.dtypes, res.dtypes], axis=1).itertuples():
+        for a, b in pd.concat([ddf._meta.dtypes, res.dtypes], axis=1).itertuples(
+            index=False
+        ):
             assert eq_dtypes(a, b)
     elif not is_dask_collection(res) and (is_index_like(res) or is_series_like(res)):
         a = ddf._meta.dtype
@@ -693,11 +710,11 @@ def valid_divisions(divisions):
     for i, x in enumerate(divisions[:-2]):
         if x >= divisions[i + 1]:
             return False
-        if isinstance(x, numbers.Number) and math.isnan(x):
+        if isinstance(x, Number) and math.isnan(x):
             return False
 
     for x in divisions[-2:]:
-        if isinstance(x, numbers.Number) and math.isnan(x):
+        if isinstance(x, Number) and math.isnan(x):
             return False
 
     if divisions[-2] > divisions[-1]:
@@ -713,3 +730,53 @@ def drop_by_shallow_copy(df, columns, errors="raise"):
         columns = [columns]
     df2.drop(columns=columns, inplace=True, errors=errors)
     return df2
+
+
+class AttributeNotImplementedError(NotImplementedError, AttributeError):
+    """NotImplementedError and AttributeError"""
+
+
+def meta_frame_constructor(like):
+    """Return a serial DataFrame constructor
+
+    Parameters
+    ----------
+    like :
+        Any series-like, Index-like or dataframe-like object.
+    """
+    if is_dask_collection(like):
+        try:
+            like = like._meta
+        except AttributeError:
+            raise TypeError(f"{type(like)} not supported by meta_frame_constructor")
+    if is_dataframe_like(like):
+        return like._constructor
+    elif is_series_like(like):
+        return like._constructor_expanddim
+    elif is_index_like(like):
+        return like.to_frame()._constructor
+    else:
+        raise TypeError(f"{type(like)} not supported by meta_frame_constructor")
+
+
+def meta_series_constructor(like):
+    """Return a serial Series constructor
+
+    Parameters
+    ----------
+    like :
+        Any series-like, Index-like or dataframe-like object.
+    """
+    if is_dask_collection(like):
+        try:
+            like = like._meta
+        except AttributeError:
+            raise TypeError(f"{type(like)} not supported by meta_series_constructor")
+    if is_dataframe_like(like):
+        return like._constructor_sliced
+    elif is_series_like(like):
+        return like._constructor
+    elif is_index_like(like):
+        return like.to_frame()._constructor_sliced
+    else:
+        raise TypeError(f"{type(like)} not supported by meta_series_constructor")
