@@ -5,7 +5,7 @@ import os.path
 from jinja2 import Environment, FileSystemLoader, Template
 from jinja2.exceptions import TemplateNotFound
 
-from ..utils import format_bytes, format_time, format_time_ago, key_split, typename
+from dask.utils import format_bytes, format_time, format_time_ago, key_split, typename
 
 FILTERS = {
     "datetime_from_timestamp": datetime.datetime.fromtimestamp,
